searchData={"items":[{"type":"module","title":"Myrmidex","doc":"A light wrapper around `StreamData`. Generate any data on the fly, or model\nthe underlying types and common fields of domain-specific structs or schemas,\noptionally using custom generators. Aimed at speeding up test setup, and\nmaximizing reusability of factory data throughout testing and development.\n\nUses reflection and composition to provide sensible defaults and allow for\nmaximum reusability in test setup, db or cache seeding, performance testing,\nor just general-purpose development utility.","ref":"Myrmidex.html"},{"type":"module","title":"Quick example - Myrmidex","doc":"Generate representative data from any term:\n\n    iex> \"🐜\"\n    ...> |> Myrmidex.many()\n    ...> |> Enum.join(\" \")\n    \"🐩 🐰 🐡 🐂 🐏 🐁 🐋 🐤 🐪 🐭 🐏 🐨 🐋 🐁 🐚 🐤\"\n\nOptionally, constrain or altogether prevent generation:\n\n    iex> \"🐜\"\n    ...> |> Myrmidex.fix()\n    ...> |> Myrmidex.many()\n    ...> |> Enum.join(\" \")\n    \"🐜 🐜 🐜 🐜 🐜 🐜 🐜 🐜 🐜 🐜\"\n\n    iex> %{species: species_generator()}\n    ...> |> Myrmidex.affix(kingdom: \"Animalia\", class: \"Insecta\")\n    ...> |> Myrmidex.many()\n    [\n      %{\n        kingdom: \"Animalia\",\n        class: \"Insecta\",\n        species: \"🪰\"\n      },\n      %{\n        kingdom: \"Animalia\",\n        class: \"Insecta\",\n        species: \"🐞\"\n      },\n      ...\n    ]","ref":"Myrmidex.html#module-quick-example"},{"type":"module","title":"More useful examples - Myrmidex","doc":"Say you're working with an api, and have defined the a schema for validation\npurposes:\n\n```elixir\ndefmodule MyApp.StripeApi.Responses.BalanceTransaction do\n  use Ecto.Schema\n\n  @primary_key {:id, :binary_id, autogenerate: true}\n  embedded_schema do\n    field :object, Ecto.Enum, values: [:balance_transaction]\n    field :amount, :integer\n    field :available_on, :utc_datetime\n    field :created, :utc_datetime\n    field :currency, :string\n    ...\n  end\n\nend\n```\n\nQuickly develop in the REPL, or write tests against this schema using\nautogenerated stream data:\n\n    iex> Mymidex.one(%BalanceTransaction{})\n    %MyApp.StripeApi.Responses.BalanceTransaction{\n      id: \"2b134329-a292-4643-b251-cf107083f6ec\",\n      object: :balance_transaction,\n      amount: 29,\n      available_on: ~U[2015-01-18 22:40:28Z],\n      created: ~U[2037-10-08 16:49:27Z],\n      currency: \"0RuOTBTOP4fYL\"\n    }\n\nThe `currency` field string generation isn't particularly useful. Let's pin\nthat value using `affix/2` and then generate a list of attrs with string keys\nwe can use to test changeset functions:\n\n    iex> %BalanceTransaction{}\n    ...> |> Myrmidex.affix(currency: \"usd\")\n    ...> |> Myrmidex.many(attr_keys: :string)\n    [\n      %{\n        \"id\" => \"7d8979b6-e99b-48d3-8acb-862866f6630a\",\n        \"amount\" => -51,\n        \"currency\" => \"usd\",\n        ...\n      },\n      %{\n        \"id\" => \"9be66e53-58e2-405a-9c7a-4ae6b536e3ce\",\n        \"amount\" => 37,\n        \"currency\" => \"usd\",\n        ...\n      },\n      ...\n    ]\n\nHelpful, but we might want to customize the binary id generation, or handle\nother specifics of this api. We can do this with a `Myrmidex.GeneratorSchema`.\n\nIdeally, generator schemas are broad enough to cover segments of the schemas\nand/or structs in our application or domain, reducing the repetition of\nad hoc test-data definitions throughout the codebase.\n\nIf we're setting up tests or using this regularly, or we need generated fields\nto build upon or be constrained by one another (e.g. the two date fields in the\nexample), we'd potentially want to go one step further and setup up a\n`Myrmidex.Factory`.\n\nFactories are also the place to define a relationship\nof a schema or set of schemas to a datastore.","ref":"Myrmidex.html#module-more-useful-examples"},{"type":"module","title":"Options - Myrmidex","doc":"Factories and genrators accept the following common set of opts:\n\n* `:generator_schema` - The default value is `Myrmidex.GeneratorSchemas.Default`.\n\n* `:limit_generation?` (`t:boolean/0`) - Constant generators in fixed map generation unless generators are affixed. The default value is `false`.\n\n* `:attr_keys` (`:string` or `:atom`) - Force the type of generated attr keys for maps and structs.\n\n* `:drop_autogenerate?` (`t:boolean/0`) - Include autogenerated ids and other fields for EctoSchemas. The default value is `false`.\n\n* `:drop_associations?` (`t:boolean/0`) - Include association fields for EctoSchemas. The default value is `false`.\n\n* `:list_opts` (`t:keyword/0`) - StreamData options for list generators. The default value is `[max_length: 30]`.","ref":"Myrmidex.html#module-options"},{"type":"function","title":"Myrmidex.affix/2","doc":"Given `term` and a compatible `term` of overrides, map values via\n`StreamData.constant/1` to override any derivation of values via\nreflection or pattern matching.\n\nWill not override previously defined generators.\n\n#","ref":"Myrmidex.html#affix/2"},{"type":"function","title":"Examples - Myrmidex.affix/2","doc":"iex> alias Myrmidex.Support.Fixtures.TestPumpkin\n    iex> humfrey = Myrmidex.affix(%TestPumpkin{}, name: \"Humfrey\", eats: \"🎃\")\n    iex> humfreys = Myrmidex.many(humfrey, 2)\n    iex> match?([%TestPumpkin{name: \"Humfrey\", eats: \"🎃\"}, %TestPumpkin{name: \"Humfrey\"}], humfreys)\n    true","ref":"Myrmidex.html#affix/2-examples"},{"type":"function","title":"Myrmidex.affix_many/2","doc":"","ref":"Myrmidex.html#affix_many/2"},{"type":"function","title":"Myrmidex.affix_many/3","doc":"Same as `affix/2` but affixes a list of `term`.","ref":"Myrmidex.html#affix_many/3"},{"type":"function","title":"Myrmidex.fix/1","doc":"Wrap any term except `%StreamData{}` in `StreamData.constant/1`.\n\n#","ref":"Myrmidex.html#fix/1"},{"type":"function","title":"Examples - Myrmidex.fix/1","doc":"iex> stream = Myrmidex.fix(\"🐜\")\n    iex> Myrmidex.many(stream, 3)\n    [\"🐜\", \"🐜\", \"🐜\"]\n    iex> match?(^stream, Myrmidex.fix(stream))\n    true","ref":"Myrmidex.html#fix/1-examples"},{"type":"function","title":"Myrmidex.many/3","doc":"Build a stream of `term`, and then emit a list of size `count` of\nrepresentative data. Like `many/2`, may also accept a range.\n\nBecause of the call to `StreamData.resize/2`, this function prevents\nnarrowing in favor of more randomly representative data.\n\n#","ref":"Myrmidex.html#many/3"},{"type":"function","title":"Examples - Myrmidex.many/3","doc":"iex> alias Myrmidex.Support.DocsGeneratorSchema\n    iex> animoji = Myrmidex.many(\"🐜\", 2..5, generator_schema: DocsGeneratorSchema)\n    iex> ascii_codes = Enum.flat_map(animoji, & String.to_charlist(&1))\n    iex> Enum.all?(ascii_codes, & &1 in 128000..128048)\n    true","ref":"Myrmidex.html#many/3-examples"},{"type":"function","title":"Myrmidex.one/2","doc":"Build a stream of `term` and then emit a random representative.\nGenerally useful with maps or structs, but can be used with any term.\n\n#","ref":"Myrmidex.html#one/2"},{"type":"function","title":"Examples - Myrmidex.one/2","doc":"iex> alias Myrmidex.Support.DocsGeneratorSchema\n    iex> animoji = Myrmidex.one(\"🐜\", generator_schema: DocsGeneratorSchema)\n    iex> [ascii_code] = String.to_charlist(animoji)\n    iex> ascii_code in 128000..128048\n    true","ref":"Myrmidex.html#one/2-examples"},{"type":"function","title":"Myrmidex.to_stream/2","doc":"The main entry point to working with stream data. Produces stream data from\n`term`, implementing inferred stream data types for fields.\n\n#","ref":"Myrmidex.html#to_stream/2"},{"type":"function","title":"Examples - Myrmidex.to_stream/2","doc":"iex> alias Myrmidex.Support.DocsGeneratorSchema\n    iex> animojis = Myrmidex.to_stream(\"🐜\", generator_schema: DocsGeneratorSchema)\n    iex> animoji = Myrmidex.one(animojis)\n    iex> [ascii_code] = String.to_charlist(animoji)\n    iex> ascii_code in 128000..128048\n    true","ref":"Myrmidex.html#to_stream/2-examples"},{"type":"function","title":"Myrmidex.via/2","doc":"","ref":"Myrmidex.html#via/2"},{"type":"behaviour","title":"Myrmidex.Factory","doc":"Defines a behaviour and utility for associating GeneratorSchemas\nwith a datasource. Typically used in your test support files as the\ndefinitive source for creating streams that make sense within\nyour domain: i.e. your schemas.\n\nOpts based to this module via `use` are persisted at compile time\nand will be passed to, e.g. `Myrmidex.to_stream/2`, in order to delegate\nto your domain-specific generator schema. This module itself uses\n`Myrmidex.GeneratorSchema`, so you may define all, or factory-specific,\ncasts within the factory module, or fallback on any generator schema.\n\nRuntime opts passed in the second argument to `to_stream/2` or\n`attrs/2` will override these module-specific compile-time opts.\n\nPutting this all together, a basic implmentation could look something\nlike:\n\n```elixir\n# test/support/factory.ex\n\ndefmodule MyApp.Support.Factory do\n  use Myrmidex.Factory\n\n  # Your factory api is up to you, but one option is to define streams\n  # that can then be composed together before calling `persist/3`.\n  def my_schema_stream(overrides \\ [], stream_opts \\ []) do\n    %MySchema{}\n    |> Myrmidex.affix(overrides)\n    |> to_stream(stream_opts)\n  end\n\n  @impl Myrmidex.Factory\n  def insert(_module, [attrs]) do\n    # Handle insert_one.\n    # This is the place to invoke your changeset functions.\n  end\n\n  def insert(_module, attrs) when is_list(attrs) do\n    # Handle insert_all.\n  end\n\n  @impl Myrmidex.GeneratorSchema\n  def cast_field({_field, _type}, _opts) do\n    # Handle and data-source specific casts: e.g. uuids, etc.\n  end\n\n  generator_schema_fallback(Myrmidex.GeneratorSchemas.Default)\nend\n```\nYour setup functions might then look like:\n\n```elixir\n# test/support/setup.ex\n\nalias MyApp.Support.Factory\n\ndef seed_my_schema(context \\ %{}) do\n  context\n  |> # do anything with context\n  |> Factory.my_schema_stream(overrides)\n  |> Factory.persist()\nend\n```\n\nDelaying the enumeration of streams leaves your factory functions\nvery composable, and useful throughout your app, during testing and\ndevelopment.","ref":"Myrmidex.Factory.html"},{"type":"callback","title":"Myrmidex.Factory.insert/2","doc":"Callback to insert one or many into a store","ref":"Myrmidex.Factory.html#c:insert/2"},{"type":"module","title":"Myrmidex.Field","doc":"A tuple representing a struct or map field with type information and, depending\non the source, a represenative term.\n\nGeneratorSchemas are passed field tuples, and cast them to generators.","ref":"Myrmidex.Field.html"},{"type":"type","title":"Myrmidex.Field.t/0","doc":"","ref":"Myrmidex.Field.html#t:t/0"},{"type":"behaviour","title":"Myrmidex.GeneratorSchema","doc":"Behaviour defining the contract for a GeneratorSchema module.\n\nGeneratorSchemas map terms, or [fields](`Myrmidex.Field`) defining the types\nof map or struct fields, into StreamData generators. See `Myrmidex.GeneratorSchemas.Default`.","ref":"Myrmidex.GeneratorSchema.html"},{"type":"callback","title":"Myrmidex.GeneratorSchema.cast/2","doc":"Callback defining custom generator mappings per term","ref":"Myrmidex.GeneratorSchema.html#c:cast/2"},{"type":"callback","title":"Myrmidex.GeneratorSchema.cast_field/2","doc":"Callback defining custom generator mappings per field--type tuple","ref":"Myrmidex.GeneratorSchema.html#c:cast_field/2"},{"type":"macro","title":"Myrmidex.GeneratorSchema.generator_schema_fallback/1","doc":"Generates code to delegate to another generator schema in the case of no\nmatching `c:cast_field/2` implementations. Should be called last in a generator\nschema module.","ref":"Myrmidex.GeneratorSchema.html#generator_schema_fallback/1"},{"type":"macro","title":"Myrmidex.GeneratorSchema.is_mappable/1","doc":"","ref":"Myrmidex.GeneratorSchema.html#is_mappable/1"},{"type":"module","title":"Myrmidex.GeneratorSchemas.Default","doc":"The default/fallback implementation of a `Myrmidex.GeneratorSchema`.\n\nDefines sensible generators for basic Elixir terms and common structs. Also\nprovides basic handling of Ecto fields for generating schemas.","ref":"Myrmidex.GeneratorSchemas.Default.html"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.atom_generator/0","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#atom_generator/0"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.binary_id_generator/0","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#binary_id_generator/0"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.boolean_generator/0","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#boolean_generator/0"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.date_generator/0","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#date_generator/0"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.datetime_generator/1","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#datetime_generator/1"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.empty_list_generator/0","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#empty_list_generator/0"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.empty_map_generator/0","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#empty_map_generator/0"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.enum_generator/1","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#enum_generator/1"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.float_generator/0","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#float_generator/0"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.id_generator/0","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#id_generator/0"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.integer_generator/0","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#integer_generator/0"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.list_generator/2","doc":"For non-empty lists, generates a list of terms representative of the original\nlist items.","ref":"Myrmidex.GeneratorSchemas.Default.html#list_generator/2"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.nil_generator/0","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#nil_generator/0"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.ref_generator/0","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#ref_generator/0"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.string_generator/0","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#string_generator/0"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.string_generator/1","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#string_generator/1"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.time_generator/0","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#time_generator/0"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.timestamp_generator/1","doc":"","ref":"Myrmidex.GeneratorSchemas.Default.html#timestamp_generator/1"},{"type":"function","title":"Myrmidex.GeneratorSchemas.Default.tuple_generator/2","doc":"Generates tuples of the same size and terms as the passed tuple.","ref":"Myrmidex.GeneratorSchemas.Default.html#tuple_generator/2"},{"type":"module","title":"Myrmidex.Helpers.StreamData","doc":"General use generators for use in generator schema modules.\n\nBy default, these generators are configured to allow for narrowing, i.e.\nuse in property-based testing. You can, for example, wrap functions in\n`StreamData.unshrinkable/1`, or define your own more suited to your domain,\nin a custom generator schema.","ref":"Myrmidex.Helpers.StreamData.html"},{"type":"function","title":"Myrmidex.Helpers.StreamData.date_stream_data/1","doc":"Generate a random date, optionally constrained before or after a given date.\n\n#","ref":"Myrmidex.Helpers.StreamData.html#date_stream_data/1"},{"type":"function","title":"Options - Myrmidex.Helpers.StreamData.date_stream_data/1","doc":"TODO: Not implemented yet.","ref":"Myrmidex.Helpers.StreamData.html#date_stream_data/1-options"},{"type":"function","title":"Myrmidex.Helpers.StreamData.datetime_stream_data/1","doc":"","ref":"Myrmidex.Helpers.StreamData.html#datetime_stream_data/1"},{"type":"function","title":"Myrmidex.Helpers.StreamData.enum_stream_data/1","doc":"Choose one from a (predetermined, limited) list of values. Note that this\ngenerator is passed via `StreamData.unshrinkable/1` to ensure random output.\n\n#","ref":"Myrmidex.Helpers.StreamData.html#enum_stream_data/1"},{"type":"function","title":"Examples - Myrmidex.Helpers.StreamData.enum_stream_data/1","doc":"iex> values = [\"🐜\", \"🪰\", \"🪳\"]\n    iex> stream = Myrmidex.Helpers.StreamData.enum_stream_data(values)\n    iex> Myrmidex.one(stream) in values\n    true","ref":"Myrmidex.Helpers.StreamData.html#enum_stream_data/1-examples"},{"type":"function","title":"Myrmidex.Helpers.StreamData.fixed_map_stream_data/2","doc":"","ref":"Myrmidex.Helpers.StreamData.html#fixed_map_stream_data/2"},{"type":"function","title":"Myrmidex.Helpers.StreamData.integer_stream_data/0","doc":"Generate integers. Given a single integer, produces a constant stream\nof that value: this is useful when composing other generators, e.g.\n`&time_stream_data/1`.\n\n#","ref":"Myrmidex.Helpers.StreamData.html#integer_stream_data/0"},{"type":"function","title":"Examples - Myrmidex.Helpers.StreamData.integer_stream_data/0","doc":"iex> stream = Myrmidex.Helpers.StreamData.integer_stream_data()\n    iex> is_integer(Myrmidex.one(stream))\n    true","ref":"Myrmidex.Helpers.StreamData.html#integer_stream_data/0-examples"},{"type":"function","title":"Myrmidex.Helpers.StreamData.integer_stream_data/1","doc":"Constrain integers generated to a given range.","ref":"Myrmidex.Helpers.StreamData.html#integer_stream_data/1"},{"type":"function","title":"Myrmidex.Helpers.StreamData.monotonic_integer_stream_data/0","doc":"Per [StreamData docs](`StreamData.repeatedly/1`).  Useful for ids, although by\ndefault ids are only unique per runtime instance: i.e. all schemas will share\nthe same sequence of ids.\n\nMore sophisticated id generators can be implemented via generator schemas.","ref":"Myrmidex.Helpers.StreamData.html#monotonic_integer_stream_data/0"},{"type":"function","title":"Myrmidex.Helpers.StreamData.string_stream_data/1","doc":"Maps binary or bitstring input into (arbitrary) matching generation ranges.\n\nMainly logical re: making the docs :) Can be replaced with your own\nimplementation with a `Myrmidex.GeneratorSchema`.\n\n#","ref":"Myrmidex.Helpers.StreamData.html#string_stream_data/1"},{"type":"function","title":"Examples - Myrmidex.Helpers.StreamData.string_stream_data/1","doc":"iex> stream = Myrmidex.Helpers.StreamData.string_stream_data(<<128>>)\n    iex> is_bitstring(Myrmidex.one(stream))\n    true\n\n    iex> stream = Myrmidex.Helpers.StreamData.string_stream_data(\"catch_all\")\n    iex> is_binary(Myrmidex.one(stream))\n    true","ref":"Myrmidex.Helpers.StreamData.html#string_stream_data/1-examples"},{"type":"function","title":"Myrmidex.Helpers.StreamData.time_stream_data/1","doc":"Generate a random time.","ref":"Myrmidex.Helpers.StreamData.html#time_stream_data/1"},{"type":"function","title":"Myrmidex.Helpers.StreamData.timestamp_stream_data/1","doc":"Defaults to current time in utc. The default generator for timestamp fields.\n\n#","ref":"Myrmidex.Helpers.StreamData.html#timestamp_stream_data/1"},{"type":"function","title":"Examples - Myrmidex.Helpers.StreamData.timestamp_stream_data/1","doc":"iex> stream = Myrmidex.Helpers.StreamData.timestamp_stream_data()\n    iex> match?(%DateTime{}, Myrmidex.one(stream))\n    true\n\n    iex> stream = Myrmidex.Helpers.StreamData.timestamp_stream_data(:utc_datetime)\n    iex> match?(%DateTime{microsecond: {0,0}}, Myrmidex.one(stream))\n    true","ref":"Myrmidex.Helpers.StreamData.html#timestamp_stream_data/1-examples"},{"type":"function","title":"Myrmidex.Helpers.StreamData.uuid_stream_data/0","doc":"Per [StreamData docs](`StreamData.repeatedly/1`).","ref":"Myrmidex.Helpers.StreamData.html#uuid_stream_data/0"},{"type":"function","title":"Myrmidex.Helpers.StreamData.via/2","doc":"Lazily transform the results of a stream via a function. I.e. just a\nshorthand for `StreamData.repeatedly/1` wrapped by `StreamData.bind/2`.","ref":"Myrmidex.Helpers.StreamData.html#via/2"},{"type":"exception","title":"Myrmidex.MissingGeneratorError","doc":"","ref":"Myrmidex.MissingGeneratorError.html"},{"type":"extras","title":"README","doc":"# Myrmidex\n\n![Tests](https://github.com/jhwls/myrmidex/actions/workflows/test.yml/badge.svg)\n![Coverage](https://github.com/jhwls/myrmidex/actions/workflows/coverage.yml/badge.svg)\n![Credo](https://github.com/jhwls/myrmidex/actions/workflows/credo.yml/badge.svg)\n[![Hex.pm](https://img.shields.io/hexpm/v/myrmidex.svg)](https://hex.pm/packages/myrmidex)\n[![Documentation](https://img.shields.io/badge/docs-grey)](https://hexdocs.pm/myrmidex)\n\nA light wrapper around `StreamData`. Generate any data on the fly, or model\nthe underlying types and common fields of domain-specific structs or schemas,\noptionally using custom generators. Aimed at speeding up test setup, and\nmaximizing reusability of factory data throughout testing and development.","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"Add to your list of test dependencies in `mix.exs`, optionally also including\nin `dev` if you want to be able to use in IEx:\n\n```elixir\ndef deps do\n  [\n    {:myrmidex, \"~> 0.2.0\", only: [:test, :dev]}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - README","doc":"Produce a stream of generically representative data from any term:\n\n```elixir\niex> \"🐜\"\n...> |> Myrmidex.many()\n...> |> Enum.join(\" \")\n\"🐩 🐰 🐡 🐂 🐏 🐁 🐋 🐤 🐪 🐭 🐏 🐨 🐋 🐁 🐚 🐤\"\n\niex> %{species: \"🐜\"}\n...> |> Myrmidex.affix(kingdom: \"Animalia\", class: \"Insecta\")\n...> |> Myrmidex.many()\n[\n  %{\n    kingdom: \"Animalia\",\n    class: \"Insecta\",\n    species: \"🪰\"\n  },\n  %{\n    kingdom: \"Animalia\",\n    class: \"Insecta\",\n    species: \"🐞\"\n  },\n  ...\n]\n\n```\n\nSee the [main module](`Myrmidex`) (`h Myrmidex`) for examples of mocking\nstructs and schemas, defining custom generator schemas, or hooking\ngeneration to persistance via factories.","ref":"readme.html#usage"},{"type":"extras","title":"Design goals - README","doc":"- Initial, usable defaults, which can be extended with domain-specific\n  concerns\n- Decouple any requirements wrt to seeding a datasource from the generation\n  of data\n- Avoid unnecessary dependencies and testing setup\n- Avoid any kind of dsl (besides that introduced by StreamData in\n  `ExUnitProperties`, which is useful for property-based testing)\n- Rely on reflection and introspection as much as possible to avoid boilerplate\n  and repetition, and also to keep factories in sync with schemas\n- Establish a robust, composable api from which to generate mock data\n  in many situations: test setup, dev setup, prototyping, etc., testing\n  changesets and validation vs testing context functions, and so on.\n- Reduce the complexity (and conversely, increase the flexibilty &\n  reusability) involved in mocking schemas with associations, or other\n  interdependencies between fields\n- Retain compatibilty with StreamData and property testing","ref":"readme.html#design-goals"},{"type":"extras","title":"Roadmap - README","doc":"- ~~Better generation of vanilla maps and lists~~\n- Reporting on generated terms\n- Better defaults for strings/emails etc.\n- Option to control shrinking\n- Option to control the default count\n- Inference/generation of ISO dates and datetimes\n- Better integration with vanilla streams","ref":"readme.html#roadmap"},{"type":"extras","title":"LICENSE","doc":"Apache License\n\nVersion 2.0, January 2004\n\nhttp://www.apache.org/licenses/\n\nTERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n1.  Definitions.\n\n    \"License\" shall mean the terms and conditions for use, reproduction,\n    and distribution as defined by Sections 1 through 9 of this document.\n\n    \"Licensor\" shall mean the copyright owner or entity authorized by\n    the copyright owner that is granting the License.\n\n    \"Legal Entity\" shall mean the union of the acting entity and all\n    other entities that control, are controlled by, or are under common\n    control with that entity. For the purposes of this definition,\n    \"control\" means (i) the power, direct or indirect, to cause the\n    direction or management of such entity, whether by contract or\n    otherwise, or (ii) ownership of fifty percent (50%) or more of the\n    outstanding shares, or (iii) beneficial ownership of such entity.\n\n    \"You\" (or \"Your\") shall mean an individual or Legal Entity\n    exercising permissions granted by this License.\n\n    \"Source\" form shall mean the preferred form for making modifications,\n    including but not limited to software source code, documentation\n    source, and configuration files.\n\n    \"Object\" form shall mean any form resulting from mechanical\n    transformation or translation of a Source form, including but\n    not limited to compiled object code, generated documentation,\n    and conversions to other media types.\n\n    \"Work\" shall mean the work of authorship, whether in Source or\n    Object form, made available under the License, as indicated by a\n    copyright notice that is included in or attached to the work\n    (an example is provided in the Appendix below).\n\n    \"Derivative Works\" shall mean any work, whether in Source or Object\n    form, that is based on (or derived from) the Work and for which the\n    editorial revisions, annotations, elaborations, or other modifications\n    represent, as a whole, an original work of authorship. For the purposes\n    of this License, Derivative Works shall not include works that remain\n    separable from, or merely link (or bind by name) to the interfaces of,\n    the Work and Derivative Works thereof.\n\n    \"Contribution\" shall mean any work of authorship, including\n    the original version of the Work and any modifications or additions\n    to that Work or Derivative Works thereof, that is intentionally\n    submitted to Licensor for inclusion in the Work by the copyright owner\n    or by an individual or Legal Entity authorized to submit on behalf of\n    the copyright owner. For the purposes of this definition, \"submitted\"\n    means any form of electronic, verbal, or written communication sent\n    to the Licensor or its representatives, including but not limited to\n    communication on electronic mailing lists, source code control systems,\n    and issue tracking systems that are managed by, or on behalf of, the\n    Licensor for the purpose of discussing and improving the Work, but\n    excluding communication that is conspicuously marked or otherwise\n    designated in writing by the copyright owner as \"Not a Contribution.\"\n\n    \"Contributor\" shall mean Licensor and any individual or Legal Entity\n    on behalf of whom a Contribution has been received by Licensor and\n    subsequently incorporated within the Work.\n\n2.  Grant of Copyright License. Subject to the terms and conditions of\n    this License, each Contributor hereby grants to You a perpetual,\n    worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n    copyright license to reproduce, prepare Derivative Works of,\n    publicly display, publicly perform, sublicense, and distribute the\n    Work and such Derivative Works in Source or Object form.\n\n3.  Grant of Patent License. Subject to the terms and conditions of\n    this License, each Contributor hereby grants to You a perpetual,\n    worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n    (except as stated in this section) patent license to make, have made,\n    use, offer to sell, sell, import, and otherwise transfer the Work,\n    where such license applies only to those patent claims licensable\n    by such Contributor that are necessarily infringed by their\n    Contribution(s) alone or by combination of their Contribution(s)\n    with the Work to which such Contribution(s) was submitted. If You\n    institute patent litigation against any entity (including a\n    cross-claim or counterclaim in a lawsuit) alleging that the Work\n    or a Contribution incorporated within the Work constitutes direct\n    or contributory patent infringement, then any patent licenses\n    granted to You under this License for that Work shall terminate\n    as of the date such litigation is filed.\n\n4.  Redistribution. You may reproduce and distribute copies of the\n    Work or Derivative Works thereof in any medium, with or without\n    modifications, and in Source or Object form, provided that You\n    meet the following conditions:\n\n    (a) You must give any other recipients of the Work or\n    Derivative Works a copy of this License; and\n\n    (b) You must cause any modified files to carry prominent notices\n    stating that You changed the files; and\n\n    (c) You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and\n    attribution notices from the Source form of the Work,\n    excluding those notices that do not pertain to any part of\n    the Derivative Works; and\n\n    (d) If the Work includes a \"NOTICE\" text file as part of its\n    distribution, then any Derivative Works that You distribute must\n    include a readable copy of the attribution notices contained\n    within such NOTICE file, excluding those notices that do not\n    pertain to any part of the Derivative Works, in at least one\n    of the following places: within a NOTICE text file distributed\n    as part of the Derivative Works; within the Source form or\n    documentation, if provided along with the Derivative Works; or,\n    within a display generated by the Derivative Works, if and\n    wherever such third-party notices normally appear. The contents\n    of the NOTICE file are for informational purposes only and\n    do not modify the License. You may add Your own attribution\n    notices within Derivative Works that You distribute, alongside\n    or as an addendum to the NOTICE text from the Work, provided\n    that such additional attribution notices cannot be construed\n    as modifying the License.\n\n    You may add Your own copyright statement to Your modifications and\n    may provide additional or different license terms and conditions\n    for use, reproduction, or distribution of Your modifications, or\n    for any such Derivative Works as a whole, provided Your use,\n    reproduction, and distribution of the Work otherwise complies with\n    the conditions stated in this License.\n\n5.  Submission of Contributions. Unless You explicitly state otherwise,\n    any Contribution intentionally submitted for inclusion in the Work\n    by You to the Licensor shall be under the terms and conditions of\n    this License, without any additional terms or conditions.\n    Notwithstanding the above, nothing herein shall supersede or modify\n    the terms of any separate license agreement you may have executed\n    with Licensor regarding such Contributions.\n\n6.  Trademarks. This License does not grant permission to use the trade\n    names, trademarks, service marks, or product names of the Licensor,\n    except as required for reasonable and customary use in describing the\n    origin of the Work and reproducing the content of the NOTICE file.\n\n7.  Disclaimer of Warranty. Unless required by applicable law or\n    agreed to in writing, Licensor provides the Work (and each\n    Contributor provides its Contributions) on an \"AS IS\" BASIS,\n    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n    implied, including, without limitation, any warranties or conditions\n    of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n    PARTICULAR PURPOSE. You are solely responsible for determining the\n    appropriateness of using or redistributing the Work and assume any\n    risks associated with Your exercise of permissions under this License.\n\n8.  Limitation of Liability. In no event and under no legal theory,\n    whether in tort (including negligence), contract, or otherwise,\n    unless required by applicable law (such as deliberate and grossly\n    negligent acts) or agreed to in writing, shall any Contributor be\n    liable to You for damages, including any direct, indirect, special,\n    incidental, or consequential damages of any character arising as a\n    result of this License or out of the use or inability to use the\n    Work (including but not limited to damages for loss of goodwill,\n    work stoppage, computer failure or malfunction, or any and all\n    other commercial damages or losses), even if such Contributor\n    has been advised of the possibility of such damages.\n\n9.  Accepting Warranty or Additional Liability. While redistributing\n    the Work or Derivative Works thereof, You may choose to offer,\n    and charge a fee for, acceptance of support, warranty, indemnity,\n    or other liability obligations and/or rights consistent with this\n    License. However, in accepting such obligations, You may act only\n    on Your own behalf and on Your sole responsibility, not on behalf\n    of any other Contributor, and only if You agree to indemnify,\n    defend, and hold each Contributor harmless for any liability\n    incurred by, or claims asserted against, such Contributor by reason\n    of your accepting any such warranty or additional liability.\n\nEND OF TERMS AND CONDITIONS","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}